#include <Keypad.h>
#include "Keyboard.h"
#include "Mouse.h"
#define speaker     2 // Speaker
const byte ROWS = 7;  // rows
const byte COLS = 7;  // columns
#define SHIFT     1   // shift key
#define FN       2   // symbol key 
#define CTRL       3   // ALT CTRL key
#define MACRO     4   // MOUSE key

#define mouseUp 0
#define mouseDown 28
#define mouseLeft 27
#define mouseRight 14 //8 9
#define mouseRb 30
#define mouseLb 1

byte colPins[COLS] = {0,1,31, 30, 27, 28, 14}; //pa13-d38,connect to the row pinouts of the keypad
byte rowPins[ROWS] = {3, 4, 5, 6, 7, 8,9}; //pb22-d30,pb23-d31,pa28-d27 connect to the column pinouts of the keypad
//define the symbols on the buttons of the keypads
char directKeys[ROWS][COLS] = {
  {KEY_ESC,'1', '2', '3', '4', '5','6'},
  {'7', '8', '9', '0', KEY_BACKSPACE,CTRL,'q'}, 
  {'w', 'e', 'r', 't', 'y','u','i'}, 
  {'o', 'p', MACRO, FN, 'a','s','d'}, 
  {'f', 'g', 'h', 'j', 'k','l',KEY_RETURN}, 
  {KEY_UP_ARROW, SHIFT, 'z', 'x', 'c','v','b'}, 
  {'n', 'm', '.', ' ', KEY_LEFT_ARROW,KEY_DOWN_ARROW,KEY_RIGHT_ARROW}
};
char shiftKeys[ROWS][COLS] = {
  {KEY_TAB,'1', '2', '3', '4', '5','6'},
  {'7', '8', '9', '0', KEY_DELETE,CTRL,'Q'},  
  {'W', 'E', 'R', 'T', 'Y','U','I'}, 
  {'O', 'P', MACRO, FN, 'A','S','D'}, 
  {'F', 'G', 'H', 'J', 'K','L',KEY_RETURN}, 
  {KEY_UP_ARROW, SHIFT, 'Z', 'X', 'C','V','B'}, 
  {'N', 'M', ',', ' ', KEY_LEFT_ARROW,KEY_DOWN_ARROW,KEY_RIGHT_ARROW}
};

char symbolKeys[ROWS][COLS] = {
   {KEY_TAB,'!', '@', '#', '$', '%','^'},
  {'&', '*', '(', ')', KEY_BACKSPACE,CTRL,'-'}, 
  {'=', '+', '[', ']', '{','}','<'}, 
  {'>', '_', MACRO, FN, 39,'~',92}, 
  {'/', '|',34, ';', ':','?',KEY_RETURN}, 
  {KEY_UP_ARROW, SHIFT, 'z' , 'x', 'c','v','b'},
  {'n', 'm', '.', ' ', KEY_LEFT_ARROW,KEY_DOWN_ARROW,KEY_RIGHT_ARROW}
};

char fnKeys[ROWS][COLS] =  {
  {KEY_F12,KEY_F1,KEY_F2, KEY_F3,KEY_F4,KEY_F5,KEY_F6},
  {KEY_F7, KEY_F8, KEY_F9, KEY_F10,KEY_F11,CTRL,'q'}, 
  {'w', 'e', 'r', 't', 'y','u','i'}, 
  {'o', 'p', MACRO, FN, 'a','s','d'}, 
  {'f', 'g', 'h', 'j', 'k','l',KEY_INSERT}, 
  {KEY_PAGE_UP, SHIFT, 'z', 'x', 'c','v','b'}, 
  {'n', 'm', '.', ' ', KEY_HOME,KEY_PAGE_DOWN,KEY_END}
};




// Create 3 new key layouts
Keypad directLayout( makeKeymap(directKeys), rowPins, colPins, sizeof(rowPins), sizeof(colPins) );
Keypad shiftLayout( makeKeymap(shiftKeys), rowPins, colPins, sizeof(rowPins), sizeof(colPins) );
Keypad fnLayout( makeKeymap(fnKeys), rowPins, colPins, sizeof(rowPins), sizeof(colPins) );
Keypad symbolLayout( makeKeymap(symbolKeys), rowPins, colPins, sizeof(rowPins), sizeof(colPins) );
byte Layout = 0;
char Key = 0;
byte State = 0;

#define ledaA 10
#define ledFn 11
#define ledCtrl 15

byte modeShift = 0;
byte modeFn = 0;
byte modeCtrl = 0;
byte modeMacro = 0;

#define ledFn 11
#define ledCtrl 15

void setup() {
  Keyboard.begin();
  Mouse.begin();
  pinMode(ledaA, OUTPUT);
  pinMode(ledFn, OUTPUT);
  pinMode(ledCtrl, OUTPUT);
  digitalWrite(ledaA, LOW);
  digitalWrite(ledFn, LOW);
  digitalWrite(ledCtrl, LOW);
  pinMode(speaker, OUTPUT);
  digitalWrite(speaker, LOW);
  playTone( 10, 1000); 
  directLayout.begin( makeKeymap(directKeys) );
  shiftLayout.begin( makeKeymap(shiftKeys) );
  fnLayout.begin( makeKeymap(fnKeys) );
  symbolLayout.begin( makeKeymap(symbolKeys) );
}

void loop() {
  getKey();
}

void resetLayout() {
modeShift = 0;
modeFn = 0;
modeCtrl = 0;
modeMacro = 0;
Key=0;
}

void respondToKey(char key, byte state) {
  switch (state) {
    case PRESSED:

        if (Layout == 1) {
        if ((key == KEY_RIGHT_ARROW)||(key == KEY_LEFT_ARROW)||(key == KEY_UP_ARROW)||(key == KEY_DOWN_ARROW)){
         Keyboard.press(KEY_LEFT_SHIFT);
        }   
        }

        if (Layout == 4){
          Keyboard.press(KEY_LEFT_CTRL);
//          if (key!=SHIFT){
//            resetLayout();            
//          }           
          }
        if (Layout == 5){
          Keyboard.press(KEY_LEFT_ALT);
//          if (key!=SHIFT){
//            resetLayout();            
//          }
          }
          
        Keyboard.press(key);
      break;


      
    case HOLD:
      break;
    case RELEASED:
      Keyboard.releaseAll();
      break;
  }
}

void getKey() {

Layout = (modeShift*1)+(modeFn*2)+(modeCtrl*4);
  
  switch (Layout) {
    case 0: // direct
      digitalWrite(ledaA, LOW); 
      digitalWrite(ledFn, LOW);
      digitalWrite(ledCtrl, LOW);
      Key = directLayout.getKey();
      State = directLayout.getState();
      break;

    case 1: // shift
      digitalWrite(ledaA, HIGH);
      digitalWrite(ledFn, LOW);
      digitalWrite(ledCtrl, LOW);
      Key = shiftLayout.getKey();
      State = shiftLayout.getState();      
      break;

    case 2: // symbol !@#$%^&*
      digitalWrite(ledaA, LOW); 
      digitalWrite(ledFn, HIGH);
      digitalWrite(ledCtrl, LOW);
      Key = symbolLayout.getKey();
      State = symbolLayout.getState();      
      break;

    case 3: // FN ... F1-F12
      digitalWrite(ledaA, HIGH);
      digitalWrite(ledFn, HIGH);
      digitalWrite(ledCtrl, LOW);
      Key = fnLayout.getKey();
      State = fnLayout.getState();
      break;

    case 4: // direct+CTRL
      digitalWrite(ledaA, LOW);
      digitalWrite(ledFn, LOW);
      digitalWrite(ledCtrl, HIGH);
      Key = directLayout.getKey();
      State = directLayout.getState();
      break;

    case 5: // direct+ALT
      digitalWrite(ledaA, HIGH);
      digitalWrite(ledFn, LOW);
      digitalWrite(ledCtrl, HIGH);
      Key = directLayout.getKey();
      State = directLayout.getState();
      break;
    case 7: // MOUSE mode
      digitalWrite(ledaA, HIGH);
      digitalWrite(ledFn, HIGH);
      digitalWrite(ledCtrl, HIGH);
//special moude mode direct keys read
pinMode (8,OUTPUT);
pinMode (9,OUTPUT);
digitalWrite(8,LOW);
digitalWrite(9,LOW);
pinMode(mouseUp,INPUT_PULLUP);
pinMode(mouseDown,INPUT_PULLUP);
pinMode(mouseLeft,INPUT_PULLUP);
pinMode(mouseRight,INPUT_PULLUP);
pinMode(mouseRb,INPUT_PULLUP);
pinMode(mouseLb,INPUT_PULLUP);
//pinMode(mouseExit,INPUT_PULLUP);

if (digitalRead(mouseUp)==LOW) // move mouse up
{
        Mouse.move(0, -1);  delay(1);
}
if (digitalRead(mouseDown)==LOW) // move mouse up
{
        Mouse.move(0, 1);  delay(1);
}
if (digitalRead(mouseLeft)==LOW) // move mouse up
{
        Mouse.move(-1,0);  delay(1);
}
if (digitalRead(mouseRight)==LOW) // move mouse up
{
        Mouse.move(1,0);  delay(1);
}
if (digitalRead(mouseLb)==LOW) // move mouse up
{
        Mouse.click(MOUSE_LEFT);
        delay(300);
        Mouse.release(MOUSE_LEFT);
}
else
{
          Mouse.release(MOUSE_LEFT);
}
if (digitalRead(mouseRb)==LOW) // move mouse up
{
        Mouse.click(MOUSE_RIGHT);
        delay(300);
        Mouse.release(MOUSE_RIGHT);
}
//  else
//  {
//          Mouse.release(MOUSE_RIGHT);
//}

      Key = shiftLayout.getKey();
      State = shiftLayout.getState(); 
if (Key==MACRO) // cancel mouse mode
{
 resetLayout();
}   
       Key = 0;
      break;

      
    default:
       modeShift = 0;
       modeFn = 0;
       modeCtrl = 0;
      break;
  }
  
  if (Key == SHIFT)
  {
    modeShift=!modeShift;
    Key = 0;
    delay(200); 
   playTone( 50, 5000); 
  }
 
  if (Key == FN)
  {
  modeFn=!modeFn;
  modeShift = 0;
  modeCtrl = 0;  
    Key = 0;
    delay(200); 
   playTone( 50, 5000); 
  }

   if (Key == CTRL)
  {
    modeCtrl=!modeCtrl;
    modeShift = 0;
    modeFn = 0;
    Key = 0;
    delay(200); 
   playTone( 50, 5000); 
  }
  
  if (Key == MACRO)
  {
//   modeMacro=!modeMacro;
  modeShift = 1;
  modeFn = 1;
  modeCtrl = 1;

    Key = 0;
    delay(200); 
    playTone( 50, 5000); 
  }

       if (Layout == 7) {
//mouse mode
        }
        else
        {
         respondToKey(Key, State);         
        }
}

void playTone(long duration, int freq) {
// duration in mSecs, frequency in hertz AHOJ PETER AKO 

//  duration *= 10;
  int period = (1.0 / freq) * 1000000;
  long elapsed_time = 0;
  int amax=0;
  byte up=1;

//    while (elapsed_time < duration) {
//      analogWrite(speaker,1020);
//      delayMicroseconds(period / 2);
//      analogWrite(speaker, 0);
//      delayMicroseconds(period / 2);
//      elapsed_time += (period);  

    while (elapsed_time < duration) {

      analogWrite(speaker,amax/2);
      delayMicroseconds(period /4); 

      analogWrite(speaker,amax);
      delayMicroseconds(period /4);

      analogWrite(speaker,amax/2) ;
      delayMicroseconds(period /4);
      
      analogWrite(speaker,0);
      delayMicroseconds(period /4);

      if (up==1) {
        if (amax < 250) {
          amax=amax+5;
        }
        else
        {
          up=0;
        }
      }
      else
      {
        if (amax > 0) amax=amax-1;
        elapsed_time += (period);
      }

  }  
      digitalWrite(speaker,LOW); 
}
